@@ -1,7 +1,8 @@
 fitstats.rma <- function(object, ..., REML) {
 
-   if (!inherits(object, "rma"))
-      stop("Argument 'object' must be an object of class \"rma\".")
+   mstyle <- .get.mstyle("crayon" %in% .packages())
+
+   .chkclass(class(object), must="rma")
 
    if (missing(REML)) {
       if (object$method == "REML") {
@@ -45,11 +46,14 @@ fitstats.rma <- function(object, ..., REML) {
 
       yis <- lapply(list(object, ...), function(x) as.vector(x$yi))
       if (!all(sapply(yis[-1], function(x) identical(x, yis[[1]]))))
-         warning("Models not all fitted to the same data.")
+         warning(mstyle$warning("Models not all fitted to the same data."), call.=FALSE)
 
    }
 
    rownames(out) <- c("logLik:", "deviance:", "AIC:", "BIC:", "AICc:")
    return(out)
 
+   #print(.fcf(out, object$digits[["fit"]]), quote=FALSE)
+   #invisible(out)
+
 }
