@@ -1,17 +1,28 @@
-plot.cumul.rma <- function(x, yaxis="tau2", xlim, ylim, xlab, ylab, at, transf, atransf, targs,
-digits, cols=c("gray90","gray10"), addgrid=TRUE, pch=19, cex=1, lwd=2, ...) {
+plot.cumul.rma <- function(x, yaxis, xlim, ylim, xlab, ylab, at, transf, atransf, targs,
+digits, cols=c("gray80","gray10"), grid=TRUE, pch=19, cex=1, lwd=2, ...) {
 
    #########################################################################
 
    mstyle <- .get.mstyle("crayon" %in% .packages())
 
-   if (!inherits(x, "cumul.rma"))
-      stop(mstyle$stop("Argument 'x' must be an object of class \"cumul.rma\"."))
+   .chkclass(class(x), must="cumul.rma")
 
-   if (is.null(x$tau2))
-      stop(mstyle$stop("Either a fixed-effects model or not sufficient data to estimate tau^2."))
+   na.act <- getOption("na.action")
 
-   yaxis <- match.arg(yaxis, c("tau2","I2","H2"))
+   if (!is.element(na.act, c("na.omit", "na.exclude", "na.fail", "na.pass")))
+      stop(mstyle$stop("Unknown 'na.action' specified under options()."))
+
+   if (missing(yaxis)) {
+      if (is.null(x$tau2)) {
+         yaxis <- "I2"
+      } else {
+         yaxis <- "tau2"
+      }
+   } else {
+      yaxis <- match.arg(yaxis, c("tau2","I2","H2"))
+      if (is.null(x$tau2))
+         stop(mstyle$stop("Cannot use yaxis=\"tau2\" for fixed-effects models."))
+   }
 
    if (missing(transf))
       transf <- FALSE
@@ -29,14 +40,22 @@ digits, cols=c("gray90","gray10"), addgrid=TRUE, pch=19, cex=1, lwd=2, ...) {
       xlab <- .setlab(x$measure, transf.char, atransf.char, gentype=2)
 
    if (missing(ylab)) {
-      if(yaxis == "tau2")
-         ylab <- "Amount of Heterogeneity (tau^2)"
-      if(yaxis == "I2")
-         ylab <- "Percentage of Variability due to Heterogeneity (I^2)"
-      if(yaxis == "H2")
-         ylab <- "Ratio of Total Variability to Sampling Variability (H^2)"
+      if (yaxis == "tau2")
+         #ylab <- "Amount of Heterogeneity (tau^2)"
+         ylab <- expression(paste("Amount of Heterogeneity ", (tau^2)))
+      if (yaxis == "I2")
+         #ylab <- "Percentage of Variability due to Heterogeneity (I^2)"
+         ylab <- expression(paste("Percentage of Variability due to Heterogeneity ", (I^2)))
+      if (yaxis == "H2")
+         #ylab <- "Ratio of Total Variability to Sampling Variability (H^2)"
+         ylab <- expression(paste("Ratio of Total Variability to Sampling Variability ", (H^2)))
    }
 
+   par.mar <- par("mar")
+   par.mar.adj <- par.mar + c(0,0.5,0,0) # need a bit more space on the right for the y-axis label
+   par(mar = par.mar.adj)
+   on.exit(par(mar = par.mar))
+
    if (missing(at))
       at <- NULL
 
@@ -44,18 +63,35 @@ digits, cols=c("gray90","gray10"), addgrid=TRUE, pch=19, cex=1, lwd=2, ...) {
       targs <- NULL
 
    if (missing(digits)) {
-      if(yaxis == "tau2")
+      if (yaxis == "tau2")
          digits <- c(2L,3L)
-      if(yaxis == "I2")
+      if (yaxis == "I2")
          digits <- c(2L,1L)
-      if(yaxis == "H2")
+      if (yaxis == "H2")
          digits <- c(2L,1L)
    } else {
       if (length(digits) == 1L)     ### digits[1] for x-axis labels
          digits <- c(digits,digits) ### digits[2] for y-axis labels
    }
 
-   ### note: digits can also be a list (e.g., digits=list(2L,3))
+   ### note: digits can also be a list (e.g., digits=list(2L,3)); trailing 0's are dropped for intergers
+
+   ddd <- list(...)
+
+   if (!is.null(ddd$addgrid))
+      grid <- ddd$addgrid
+
+   ### grid argument can either be a logical or a color
+
+   if (is.logical(grid))
+      gridcol <- "lightgray"
+   if (is.character(grid)) {
+      gridcol <- grid
+      grid <- TRUE
+   }
+
+   lplot <- function(..., addgrid) plot(...)
+   laxis <- function(..., addgrid) axis(...)
 
    #########################################################################
 
@@ -70,9 +106,13 @@ digits, cols=c("gray90","gray10"), addgrid=TRUE, pch=19, cex=1, lwd=2, ...) {
    if (yaxis == "H2")
       dat$yval <- x$H2
 
-   ### remove any rows with NAs
+   ### apply chosen na.action
 
-   dat <- na.omit(dat)
+   if (na.act == "na.fail" && anyNA(dat))
+      stop(mstyle$stop("Missing values in results."))
+
+   if (na.act == "na.omit")
+      dat <- na.omit(dat)
 
    ### number of remaining rows/points
 
@@ -91,29 +131,29 @@ digits, cols=c("gray90","gray10"), addgrid=TRUE, pch=19, cex=1, lwd=2, ...) {
    ### set xlim and ylim values
 
    if (missing(xlim)) {
-      xlim <- range(dat$estim)
+      xlim <- range(dat$estim, na.rm=TRUE)
    } else {
       xlim <- sort(xlim) ### just in case the user supplies the limits in the wrong order
    }
 
    if (missing(ylim)) {
-      ylim <- range(dat$yval)
+      ylim <- range(dat$yval, na.rm=TRUE)
    } else {
       ylim <- sort(ylim) ### just in case the user supplies the limits in the wrong order
    }
 
    ### if user has specified 'at' argument, make sure xlim actually contains the min and max 'at' values
 
-   if(!is.null(at)) {
+   if (!is.null(at)) {
       xlim[1] <- min(c(xlim[1], at), na.rm=TRUE)
       xlim[2] <- max(c(xlim[2], at), na.rm=TRUE)
    }
 
    ### set up plot
 
-   plot(NA, NA, xlim=xlim, ylim=ylim, xlab=xlab, ylab=ylab, xaxt="n", yaxt="n")
+   lplot(NA, NA, xlim=xlim, ylim=ylim, xlab=xlab, ylab=ylab, xaxt="n", yaxt="n", ...)
 
-   ### generate x axis positions if none are specified
+   ### generate x-axis positions if none are specified
 
    if (is.null(at)) {
       at <- axTicks(side=1)
@@ -126,35 +166,35 @@ digits, cols=c("gray90","gray10"), addgrid=TRUE, pch=19, cex=1, lwd=2, ...) {
 
    if (is.function(atransf)) {
       if (is.null(targs)) {
-         at.lab <- formatC(sapply(at.lab, atransf), digits=digits[[1]], format="f", drop0trailing=ifelse(class(digits[[1]]) == "integer", TRUE, FALSE))
+         at.lab <- formatC(sapply(at.lab, atransf), digits=digits[[1]], format="f", drop0trailing=is.integer(digits[[1]]))
       } else {
-         at.lab <- formatC(sapply(at.lab, atransf, targs), digits=digits[[1]], format="f", drop0trailing=ifelse(class(digits[[1]]) == "integer", TRUE, FALSE))
+         at.lab <- formatC(sapply(at.lab, atransf, targs), digits=digits[[1]], format="f", drop0trailing=is.integer(digits[[1]]))
       }
    } else {
-      at.lab <- formatC(at.lab, digits=digits[[1]], format="f", drop0trailing=ifelse(class(digits[[1]]) == "integer", TRUE, FALSE))
+      at.lab <- formatC(at.lab, digits=digits[[1]], format="f", drop0trailing=is.integer(digits[[1]]))
    }
 
    ### add x-axis
 
-   axis(side=1, at=at, labels=at.lab)
+   laxis(side=1, at=at, labels=at.lab, ...)
 
    ### add y-axis
 
    aty <- axTicks(side=2)
-   axis(side=2, at=aty, labels=formatC(aty, digits=digits[[2]], format="f", drop0trailing=ifelse(class(digits[[2]]) == "integer", TRUE, FALSE)))
+   laxis(side=2, at=aty, labels=formatC(aty, digits=digits[[2]], format="f", drop0trailing=is.integer(digits[[2]])), ...)
 
    ### add grid
 
-   if (addgrid) {
-      abline(v=at, lty="dotted", col="lightgray")
-      abline(h=aty, lty="dotted", col="lightgray")
+   if (.isTRUE(grid)) {
+      abline(v=at,  lty="dotted", col=gridcol)
+      abline(h=aty, lty="dotted", col=gridcol)
    }
 
    ### vector with color gradient for points
 
    cols.points <- colorRampPalette(cols)(k)
 
-   #gray.vals.points <- seq(from=.9, to=.1, length=k)
+   #gray.vals.points <- seq(from=.9, to=.1, length.out=k)
    #cols.points <- gray(gray.vals.points)
    #cols <- colorRampPalette(c("yellow","red"))(k)
    #cols <- colorRampPalette(c("blue","red"))(k)
@@ -166,6 +206,8 @@ digits, cols=c("gray90","gray10"), addgrid=TRUE, pch=19, cex=1, lwd=2, ...) {
    ### looks better this way, especially when k is low
 
    for (i in seq_len(k-1)) {
+      if (is.na(dat$estim[i]) || is.na(dat$estim[i+1]) || is.na(dat$yval[i]) || is.na(dat$yval[i+1]))
+         next
       estims <- approx(c(dat$estim[i], dat$estim[i+1]), n=50)$y
       yvals  <- approx(c(dat$yval[i], dat$yval[i+1]), n=50)$y
       cols.lines <- colorRampPalette(c(cols.points[i], cols.points[i+1]))(50)
@@ -179,7 +221,7 @@ digits, cols=c("gray90","gray10"), addgrid=TRUE, pch=19, cex=1, lwd=2, ...) {
 
    ### add points
 
-   points(dat$estim, dat$yval, pch=pch, col=cols.points, cex=cex)
+   points(x=dat$estim, y=dat$yval, pch=pch, col=cols.points, cex=cex)
 
    ### redraw box around plot
 
@@ -187,6 +229,8 @@ digits, cols=c("gray90","gray10"), addgrid=TRUE, pch=19, cex=1, lwd=2, ...) {
 
    ### return data frame invisibly
 
+   dat$col <- cols.points
+
    invisible(dat)
 
 }
