@@ -1,11 +1,13 @@
 ### library(metafor); library(testthat); Sys.setenv(NOT_CRAN="true")
 
-### see also: http://www.metafor-project.org/doku.php/plots:funnel_plot_with_trim_and_fill
+### see also: https://www.metafor-project.org/doku.php/plots:funnel_plot_with_trim_and_fill
 
 context("Checking plots example: funnel plot with trim and fill")
 
 test_that("plot can be drawn.", {
 
+   expect_equivalent(TRUE, TRUE) # avoid 'Empty test' message
+
    skip_on_cran()
 
    opar <- par(no.readonly=TRUE)
@@ -23,7 +25,7 @@ test_that("plot can be drawn.", {
    taf <- trimfill(res)
 
    ### draw funnel plot with missing studies filled in
-   funnel(taf)
+   funnel(taf, legend=TRUE)
 
    par(opar)
 
